package com.dianmi.service.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import com.dianmi.dto.QueryWorkflowDetailDto;
import com.dianmi.dto.QueryWorkflowTypeDto;
import com.dianmi.service.IOpenOAService;
import com.dianmi.ws.ArrayOfString;
import com.dianmi.ws.ArrayOfWorkflowBaseInfo;
import com.dianmi.ws.WorkflowBaseInfo;
import com.dianmi.ws.WorkflowRequestInfo;
import com.dianmi.ws.WorkflowService;
import com.dianmi.ws.WorkflowServicePortType;

import lombok.extern.slf4j.Slf4j;
@Slf4j
@Service
public class OpenOAServiceImpl implements IOpenOAService,InitializingBean {

	@Value("${oa.baseUrl}")
	private String baseUrl;
	@Value("${oa.appId}")
	private String appId;
	private String oaWsdlUrl;
	private WorkflowServicePortType workflowServicePortType;
	
	private RestTemplate restTemplate;
	@Override
	public List<WorkflowBaseInfo> getCreateWorkflowTypeList(QueryWorkflowTypeDto dto) {
		String conditions = dto.getConditions();
		String[] str = null;
		if (StringUtils.isNotEmpty(conditions)) {
			  str = conditions.split(",");
		}
		ArrayOfString aos = new ArrayOfString();
		if (ArrayUtils.isNotEmpty(str)) {
			aos.getString().addAll(new ArrayList<String>(Arrays.asList(str)));
		}
		ArrayOfWorkflowBaseInfo bas = workflowServicePortType.getCreateWorkflowTypeList(dto.getPageNo(), 
				dto.getPageSize(),
				dto.getRecordCount(), dto.getUserid(), aos);
		return bas.getWorkflowBaseInfo();
	}

	@Override
	public WorkflowRequestInfo getWorkflowRequest(QueryWorkflowDetailDto dto) {
		 
		return workflowServicePortType.getWorkflowRequest(dto.getRequestid(), dto.getUserid(), dto.getFromrequestid());
	}
	@Override
	public void afterPropertiesSet() {
		try {
			oaWsdlUrl = baseUrl+"/services/WorkflowService?wsdl";
			WorkflowService workflowService = new WorkflowService(new URL(oaWsdlUrl));
			workflowServicePortType = workflowService.getWorkflowServiceHttpPort();
			restTemplate = new RestTemplate();
		} catch (MalformedURLException e) {
			log.error("请求oa流程接口异常：{}",e.getMessage(),e);
		}
 
	}
    
	@Override
	public void redirectOA(Map<String, String> paraMap) {
		 
		
	}
}
